# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from __future__ import absolute_import

import tempfile

from django.conf import settings
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.template.backends.base import BaseEngine
from django.template.backends.utils import csrf_input_lazy, csrf_token_lazy
from django.template.context import _builtin_context_processors
from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from mako import exceptions as mako_exceptions
from mako.lookup import TemplateLookup as MakoTemplateLookup
from mako.template import Template as MakoTemplate


class MakoTemplates(BaseEngine):
    app_dirname = settings.MAKO_DIR_NAME

    def __init__(self, params):
        params = params.copy()
        options = params.pop("OPTIONS").copy()
        super(MakoTemplates, self).__init__(params)

        # Defaut values for initializing the MakoTemplateLookup class
        # You can define them in the backend OPTIONS dict.
        options.setdefault("directories", self.template_dirs)
        options.setdefault("module_directory", tempfile.gettempdir())
        options.setdefault("input_encoding", "utf-8")
        options.setdefault("output_encoding", "utf-8")
        options.setdefault("encoding_errors", "replace")
        options.setdefault("collection_size", 500)
        options.setdefault(
            "default_filters",
            settings.MAKO_DEFAULT_FILTERS
            if hasattr(settings, "MAKO_DEFAULT_FILTERS")
            else [],
        )

        # Use context processors like Django
        context_processors = options.pop("context_processors", [])
        self.context_processors = context_processors

        # Use the mako template lookup class to find templates
        self.lookup = MakoTemplateLookup(**options)

    @cached_property
    def template_context_processors(self):
        context_processors = _builtin_context_processors
        context_processors += tuple(self.context_processors)
        return tuple(import_string(path) for path in set(context_processors))

    def from_string(self, template_code):
        try:
            return Template(MakoTemplate(template_code, lookup=self.lookup), [])
        except mako_exceptions.SyntaxException as err:
            raise TemplateSyntaxError(err.args)

    def get_template(self, template_name):
        try:
            return Template(
                self.lookup.get_template(template_name),
                self.template_context_processors,
            )
        except mako_exceptions.TemplateLookupException as err:
            raise TemplateDoesNotExist(err.args)
        except mako_exceptions.CompileException as err:
            raise TemplateSyntaxError(err.args)


class Template(object):
    def __init__(self, template, context_processors):
        self.template = template
        self.context_processors = context_processors

    def render(self, context=None, request=None):
        if context is None:
            context = {}

        if request is not None:
            for processor in self.context_processors:
                try:
                    context.update(processor(request))
                except Exception:  # pylint: disable=broad-except
                    pass

            context["request"] = request
            context["csrf_input"] = csrf_input_lazy(request)
            context["csrf_token"] = csrf_token_lazy(request)

        return self.template.render_unicode(**context)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from blueapps.core.exceptions import ParamValidationError
from blueapps.utils.time import strftime_local


class MultipleIntField(serializers.Field):
    """
    用于整数多选查询的字段
    """

    def to_internal_value(self, data):  # pylint: disable=no-self-use
        """
        Transform the *incoming* primitive data into a native value.
        """
        ret = []
        for value in data.split(","):
            try:
                ret.append(int(value))
            except ValueError:
                raise ParamValidationError(_("请求参数必须全部为整数"))
        return ret

    def to_representation(self, value):  # pylint: disable=no-self-use
        """
        Transform the *outgoing* native value into primitive data.
        """
        return value


class MultipleStrField(serializers.Field):
    """
    用于字符串多选查询的字段
    """

    def to_internal_value(self, data):  # pylint: disable=no-self-use
        """
        Transform the *incoming* primitive data into a native value.
        """
        data = [str(value) for value in data.split(",")]
        return data

    def to_representation(self, value):  # pylint: disable=no-self-use
        """
        Transform the *outgoing* native value into primitive data.
        """
        return value


class CustomDateTimeField(serializers.DateTimeField):
    """
    自定义数据库时间字段
    """

    def to_representation(self, value):  # pylint: disable=no-self-use
        """
        Transform the *outgoing* native value into primitive data.
        """
        if not value:
            return None

        return strftime_local(value, fmt=settings.REST_FRAMEWORK.get("DATETIME_FORMAT", "%Y-%m-%d %H:%M:%S%z"),)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import abc
import os
import sys

from django.core.exceptions import ValidationError


class BaseValidator(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def validate(self, *args, **kwargs):
        """
        对应的校验逻辑
        """


class EnvValidator(BaseValidator):
    ENV_SETTINGS = {
        "open_v2": ("BK_PAAS_HOST", "APP_ID", "APP_TOKEN"),
        "open_v3": (
            "BK_PAAS2_URL",
            "BKPAAS_MAJOR_VERSION",
            "BKPAAS_APP_ID",
            "BKPAAS_APP_SECRET",
            "BK_COMPONENT_API_URL",
        ),
        "ieod": (
            "BKPAAS_APP_ID",
            "BKPAAS_APP_SECRET",
            "BKPAAS_LOGIN_URL",
            "BKPAAS_REMOTE_STATIC_URL",
            "BKPAAS_APIGW_OAUTH_API_URL",
        ),
        "tencent": (
            "BKPAAS_APP_ID",
            "BKPAAS_APP_SECRET",
            "BKPAAS_LOGIN_URL",
            "BKPAAS_REMOTE_STATIC_URL",
            "BKPAAS_APIGW_OAUTH_API_URL",
        ),
    }
    VALID_RUN_VER = ("ieod", "open", "tencent")

    def __init__(self, run_ver):
        if run_ver not in self.VALID_RUN_VER:
            raise ValidationError(f'run_ver值不合法，合法的值有：{",".join(self.VALID_RUN_VER)}')

        self.run_ver = run_ver

    def _check_env(self, env_settings_key):
        return [
            env for env in self.ENV_SETTINGS[env_settings_key] if os.getenv(env) is None
        ]

    def validate(self, *args, **kwargs):
        needed_envs = []

        if self.run_ver in self.ENV_SETTINGS:
            needed_envs = self._check_env(self.run_ver)
        elif self.run_ver == "open":
            env_settings_key = (
                "open_v3"
                if int(os.getenv("BKPAAS_MAJOR_VERSION", 2)) == 3
                else "open_v2"
            )
            needed_envs = self._check_env(env_settings_key)

        if needed_envs:
            sys.stderr.write(
                f"当前运行环境缺少部分需要的环境变量，可能导致项目运行报错，"
                f'请确认以下环境变量已经配置并赋值: {",".join(needed_envs)}\n'
            )
            if self.run_ver == "open":
                sys.stderr.write("注意，当前版本为社区版环境，请确认 BKPAAS_MAJOR_VERSION 已经配置并正确赋值\n")



# -*- coding: utf-8 -*-
from celery import Task, current_app
from celery.schedules import maybe_schedule


class BlueappsPeriodicTask(Task):
    """A task that adds itself to the :setting:`beat_schedule` setting. 兼容 celery4 & celery5"""

    abstract = True
    ignore_result = True
    relative = False
    options = None
    compat = True

    def __init__(self):
        if not hasattr(self, "run_every"):
            raise NotImplementedError("Periodic tasks must have a run_every attribute")
        self.run_every = maybe_schedule(self.run_every, self.relative)
        super(BlueappsPeriodicTask, self).__init__()

    @classmethod
    def on_bound(cls, app):
        app.conf.beat_schedule[cls.name] = {
            "task": cls.name,
            "schedule": cls.run_every,
            "args": (),
            "kwargs": {},
            "options": cls.options or {},
            "relative": cls.relative,
        }


def periodic_task(*args, **options):
    """Deprecated decorator, please use :setting:`beat_schedule`."""
    return current_app.task(**dict({"base": BlueappsPeriodicTask}, **options))



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from django.contrib.auth.backends import ModelBackend

from blueapps import metrics
from blueapps.account import get_user_model
from blueapps.account.conf import ConfFixture
from blueapps.account.utils.http import send
from blueapps.utils.tools import resolve_login_url

logger = logging.getLogger("component")

TOKEN_TYPE = "bk_ticket"


class TicketBackend(ModelBackend):
    def authenticate(self, request=None, bk_ticket=None):
        logger.debug("进入 Paas 认证 Backend")
        if not bk_ticket:
            return None

        result, user_info = self.verify_bk_ticket(bk_ticket, request)
        if not result:
            return None

        user_model = get_user_model()
        user, _ = user_model.objects.get_or_create(username=user_info["username"])
        user.nickname = user_info["username"]
        user.avatar_url = user_info["avatar_url"]
        user.save()
        return user

    @staticmethod
    def verify_bk_ticket(bk_ticket, request=None):
        """
        验证 OA 登录票据
        @param {string} bk_ticket OA 登录票据
        @return {tuple} ret
        @return {boolean} ret[0] 是否认证通过
        @return {dict} ret[1] 当 result=True，该字段为用户信息，举例
            {
                'username': 'tester',
                'avatar_url': 'http://???.??.com/avatars/tester/avatar.jpg'
            }
        """
        api_params = {"bk_ticket": bk_ticket}
        try:
            with metrics.observe(
                metrics.BLUEAPPS_USER_TOKEN_VERIFY_DURATION, hostname=metrics.HOSTNAME, token_type=TOKEN_TYPE
            ):
                response = send(
                    resolve_login_url(ConfFixture.USER_INFO_URL, request, "http"),
                    "GET",
                    api_params,
                )
            ret = response.get("ret")

            if ret == 0:
                return True, response["data"]
            else:
                metrics.BLUEAPPS_USER_TOKEN_VERIFY_FAILED_TOTAL.labels(
                    hostname=metrics.HOSTNAME, token_type=TOKEN_TYPE, err="verify_fail"
                ).inc()
                logger.error("bk_ticket 验证失败，error={}，ret={}".format(response["msg"], ret))
                return False, None
        except Exception:  # pylint: disable=broad-except
            metrics.BLUEAPPS_USER_TOKEN_VERIFY_FAILED_TOTAL.labels(
                hostname=metrics.HOSTNAME, token_type=TOKEN_TYPE, err="unknow_execption_raise"
            ).inc()
            logger.exception("bk_ticket 验证异常")
            return False, None



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸 (Blueking) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at https://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and limitations under the License.
"""

import json
import logging
import re

from drf_yasg import openapi
from drf_yasg.inspectors import SwaggerAutoSchema
from drf_yasg.utils import merge_params

logger = logging.getLogger(__name__)


class AnnotationAutoSchema(SwaggerAutoSchema):
    params_regex = re.compile("^param: (?P<key>.*?): (?P<infos>.*?)$")
    response_regex = re.compile("^return: (?P<return_desc>.*?)$")
    body_regex = re.compile("^body: (?P<body_name>.*?)$")
    dict_string_regex = re.compile("^(?P<field_desc>.*?)[(](?P<field_type>.*?)[)]$")
    key_required_regex = re.compile("^(?P<field_desc>.*?)[(](?P<field_require>.*?)[)]$")

    def get_operation(self, operation_keys=None):
        operation = super().get_operation(operation_keys)
        if not hasattr(operation, "summary"):
            operation.summary = operation.description
        parameters, response = self._extract_params_and_returns()
        operation.parameters = merge_params(operation.parameters, parameters)
        if response and not self.overrides.get("responses"):
            operation.responses["200"] = response
        return operation

    def _get_schema_based_on_data(self, data):
        """将对应类型数据转化为Schema对象"""
        if isinstance(data, dict):
            required_properties = []
            properties = {}
            for key, info in data.items():
                key = str(key)
                match_result = self.key_required_regex.match(key)
                if match_result and match_result["field_require"] == "required":
                    key = match_result["field_desc"]
                    required_properties.append(key)
                properties[key] = self._get_schema_based_on_data(info)
            return openapi.Schema(type=openapi.TYPE_OBJECT, properties=properties, required=required_properties,)
        elif isinstance(data, list):
            items = (
                self._get_schema_based_on_data(data[0]) if len(data) > 0 else openapi.Schema(type=openapi.TYPE_ARRAY)
            )
            return openapi.Schema(type=openapi.TYPE_ARRAY, items=items)
        elif isinstance(data, str):
            match_result = self.dict_string_regex.match(data)
            desc = match_result["field_desc"] if match_result else data
            field_type = match_result["field_type"] if match_result else openapi.TYPE_STRING
            return openapi.Schema(description=desc, type=field_type)
        else:
            raise TypeError("data {} should be the type of dict or list or string".format(data))

    def _extract_params_and_returns(self, description=None):
        """提取符合格式的接口注释中相关的参数和相应信息，加入对应的参数和响应列表"""
        if description is None:
            description = self.view.get_view_description()
        parameters = []
        response = None
        response_name = ""
        collecting_response_lines = False
        response_lines = []
        body_name = ""
        body_lines = []
        collecting_body_lines = False
        for line in description.splitlines():
            # 处理普通参数
            match_param = self.params_regex.match(line)
            if match_param:
                try:
                    param_key = match_param["key"]
                    param_infos = match_param["infos"].strip()
                    param_required = True if "required" in param_infos else False
                    param_infos = param_infos.split(",")
                    if param_required:
                        param_infos = param_infos[:-1]
                    param_desc, param_type, param_in = (
                        ",".join(param_infos[:-2]),
                        param_infos[-2].strip(),
                        param_infos[-1].strip(),
                    )
                    openapi_param = openapi.Parameter(
                        name=param_key, in_=param_in, description=param_desc, type=param_type, required=param_required,
                    )
                    parameters.append(openapi_param)
                except Exception as e:
                    logger.error(
                        "[AnnotationAutoSchema]: transform param error: {} with annotation: {}".format(e, line)
                    )
                    raise e

            # 处理响应
            match_return = self.response_regex.match(line)
            if match_return:
                response_name = match_return["return_desc"]
                collecting_response_lines = True
                continue

            # 处理body参数情况
            match_body = self.body_regex.match(line)
            if match_body:
                body_name = match_body["body_name"] or "data"
                collecting_body_lines = True
                continue

            if collecting_response_lines:
                response_lines.append(line.strip())
            if collecting_body_lines:
                body_lines.append(line.strip())
            if line == "":
                collecting_response_lines = False
                collecting_body_lines = False

        if response_lines:
            try:
                response_data = json.loads("".join(response_lines))
            except Exception as e:
                logger.error(
                    "[AnnotationAutoSchema]: json loads response_data error: {}, response_data: {})".format(
                        e, "".join(response_lines)
                    )
                )
                raise e
            response = openapi.Response(
                description=response_name, schema=self._get_schema_based_on_data(response_data),
            )

        if body_lines:
            try:
                body_data = json.loads("".join(body_lines))
            except Exception as e:
                logger.error(
                    "[AnnotationAutoSchema]: json loads body_data error: {}, body_data: {})".format(
                        e, "".join(body_lines)
                    )
                )
                raise e

            body_param = openapi.Parameter(
                name=body_name, in_=openapi.IN_BODY, schema=self._get_schema_based_on_data(body_data),
            )
            parameters.append(body_param)

        return parameters, response

    def should_filter(self):
        if self.overrides.get("ignore_filter_query"):
            return False
        return super(AnnotationAutoSchema, self).should_filter()



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import json
import logging
import traceback
from urllib.parse import urlparse, urlunparse

import requests
from django.http import QueryDict
from django.shortcuts import resolve_url
from django.utils.translation import gettext_lazy as _

from blueapps.core.exceptions.base import ApiNetworkError, ApiResultError

logger = logging.getLogger("component")


def send(url, method, params, timeout=None, **kwargs):
    """
    统一请求处理，定制化参数， GET 参数使用 form-data，POST 参数使用 json 字符串，返回内容
    要求为 JSON 格式

    @exception
        ApiResultError：非 JSON 返回，抛出 ApiResultError
        ApiNetworkError： 请求服务端超时

    @param url：string，请求 URL
    @param method：string，请求方法，目前仅支持 GET、POST
    @param params：dict，请求参数 KV 结构
    @param timeout: float，服务器在 timeout 秒内没有应答，将会引发一个异常
    """
    session = requests.session()

    try:
        if method.upper() == "GET":
            response = session.request(
                method="GET", url=url, params=params, timeout=timeout, **kwargs
            )
        elif method.upper() == "POST":
            session.headers.update({"Content-Type": "application/json; chartset=utf-8"})
            response = session.request(
                method="POST",
                url=url,
                data=json.dumps(params),
                timeout=timeout,
                **kwargs
            )
        else:
            raise Exception(_(u"异常请求方式，%s") % method)
    except requests.exceptions.Timeout:
        err_msg = _(u"请求超时，url=%s，method=%s，params=%s，timeout=%s") % (
            url,
            method,
            params,
            timeout,
        )
        raise ApiNetworkError(err_msg)

    logger.debug(
        "请求记录, url={}, method={}, params={}, response={}".format(
            url, method, params, response
        )
    )

    if response.status_code != requests.codes.ok:
        err_msg = _(u"返回异常状态码，status_code=%s，url=%s，method=%s，" u"params=%s") % (
            response.status_code,
            url,
            method,
            json.dumps(params),
        )
        raise ApiResultError(err_msg)

    try:
        return response.json()
    except Exception:  # pylint: disable=broad-except
        err_msg = _(
            u"返回内容不符合 JSON 格式，url=%s，method=%s，params=%s，error=%s，" u"response=%s"
        ) % (
            url,
            method,
            json.dumps(params),
            traceback.format_exc(),
            response.text[:1000],
        )
        raise ApiResultError(err_msg)


def build_redirect_url(next_url, current_url, redirect_field_name, extra_args=None):
    """
    即将访问的 CUR_URL 页面， 加上下一步要跳转的 NEXT 页面
    @param {string} next_url 页面链接，比如 http://a.com/page1/
    @param {string} current_url
    """
    resolved_url = resolve_url(current_url)

    login_url_parts = list(urlparse(resolved_url))

    querystring = QueryDict(login_url_parts[4], mutable=True)
    querystring[redirect_field_name] = next_url

    if extra_args:
        querystring.update(extra_args)

    login_url_parts[4] = querystring.urlencode(safe="/")

    return urlunparse(login_url_parts)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""
import datetime
import time

import pytz
from django.utils import timezone

# 默认时间戳乘数
DEFAULT_MULTIPLICATOR = 1

SHOW_TZ = False
FMT_LENGTH = None if SHOW_TZ else 16


def timeformat_to_timestamp(timeformat, time_multiplicator=DEFAULT_MULTIPLICATOR):
    """
    时间格式 -> 时间戳
    :param {str} timeformat: 日期格式字符串
    :param {int} time_multiplicator: 时间倍数
    :return: 时间戳
    """
    if not timeformat:
        return None
    if type(timeformat) in [str]:
        # 时间字符串转时间戳
        timestamp = int(time.mktime(time.strptime(timeformat, "%Y-%m-%d %H:%M:%S")))
    else:
        # datetime 转时间戳
        timestamp = int(timeformat.strftime("%s"))
    return int(timestamp * time_multiplicator)


def timestamp_to_datetime(from_timestamp, time_multiplicator=DEFAULT_MULTIPLICATOR):
    """
    时间戳 -> datetime
    :param {int} from_timestamp: 时间戳
    :param {int} time_multiplicator: 时间倍数
    :return: 时间戳
    """
    utc_tz = pytz.timezone("UTC")
    utc_dt = utc_tz.localize(
        datetime.datetime.utcfromtimestamp(int(from_timestamp) / time_multiplicator)
    )
    return utc_dt


def format_datetime(o_datetime):
    """
    格式化日志对象展示格式

    :param {datetime} o_datetime datetime实例
    :return 格式化时间字符串
    """
    return o_datetime.strftime("%Y-%m-%d %H:%M:%S%z")


def strftime_local(aware_time, fmt="%Y-%m-%d %H:%M:%S"):
    """
    格式化aware_time为本地时间
    :param {datetime} aware_time 时间字段
    :param {string} fmt 时间format格式
    :return 格式化时间字符串
    """
    if not aware_time:
        # 当时间字段允许为NULL时，直接返回None
        return None
    if timezone.is_aware(aware_time):
        # translate to time in local timezone
        aware_time = timezone.localtime(aware_time)
    return aware_time.strftime(fmt)


def localtime_to_timezone(d_time, to_zone):
    """
    将时间字符串根据源时区转为用户时区
    :param {datetime} d_time 时间
    :param {string} to_zone 时区
    :return 用户时区时间
    """
    zone = pytz.timezone(to_zone)
    return d_time.astimezone(zone)


def time_to_string(date_time):
    """
    传入一个标准时间，返回其字符串形式
    :param {datetime} date_time: 时间
    :return: 时间字符串
    """
    return date_time.strftime("%Y-%m-%d %H:%M:%S")


def date_to_string(_data):
    """
    传入一个标准日期，返回其字符串形式
    :param {datetime} _data: 日期
    :return: 日期字符串
    """
    return _data.strftime("%Y-%m-%d")


def string_to_time(t_str):
    """
    传入一个字符串，返回其标准时间格式
    :param {string} t_str: 时间字符串
    :return: 时间
    """
    return datetime.datetime.strptime(t_str, "%Y-%m-%d %H:%M:%S")


def string_to_date(d_str):
    """
    传入一个字符串，返回其标准日期格式
    :param {string} d_str: 日期字符串
    :return: 日期
    """
    return datetime.datetime.strptime(d_str, "%Y-%m-%d")



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from django.conf import settings
from django.contrib import auth
from django.core.cache import caches

from blueapps.account.components.bk_token.forms import AuthenticationForm
from blueapps.account.conf import ConfFixture
from blueapps.account.handlers.response import ResponseHandler
from blueapps.utils import client

try:
    from django.utils.deprecation import MiddlewareMixin
except Exception:  # pylint: disable=broad-except
    MiddlewareMixin = object


logger = logging.getLogger("component")
cache = caches["login_db"]


class LoginRequiredMiddleware(MiddlewareMixin):
    def process_view(self, request, view, args, kwargs):
        """
        Login paas by two ways
        1. views decorated with 'login_exempt' keyword
        2. User has logged in calling auth.login
        """
        if hasattr(request, "is_wechat") and request.is_wechat():
            return None

        if hasattr(request, "is_bk_jwt") and request.is_bk_jwt():
            return None

        if hasattr(request, "is_rio") and request.is_rio():
            return None

        if getattr(view, "login_exempt", False):
            return None

        user = self.authenticate(request)
        if user:
            return None

        handler = ResponseHandler(ConfFixture, settings)

        # check if the user is forbidden
        user_forbidden, msg = self.is_user_forbidden(request)
        if user_forbidden:
            return handler.build_403_response(msg)

        return handler.build_401_response(request)

    def process_response(self, request, response):
        return response

    def authenticate(self, request):
        form = AuthenticationForm(request.COOKIES)
        if not form.is_valid():
            return None

        bk_token = form.cleaned_data["bk_token"]
        session_key = request.session.session_key
        if session_key:
            # 确认 cookie 中的 ticket 和 cache 中的是否一致
            cache_session = cache.get(session_key)
            is_match = cache_session and bk_token == cache_session.get("bk_token")
            if is_match and request.user.is_authenticated:
                return request.user

        user = auth.authenticate(request=request, bk_token=bk_token)
        if user is not None and user.username != request.user.username:
            auth.login(request, user)

        if user is not None and request.user.is_authenticated:
            # 登录成功，重新调用自身函数，即可退出
            cache.set(session_key, {"bk_token": bk_token}, settings.LOGIN_CACHE_EXPIRED)
            return self.authenticate(request)
        return user

    @staticmethod
    def is_user_forbidden(request) -> (bool, str):
        """
        check if the user is forbidden
        """
        USER_FORBIDDEN_CODE = "1302403"
        form = AuthenticationForm(request.COOKIES)
        if not form.is_valid():
            return False, ""

        bk_token = form.cleaned_data["bk_token"]
        try:
            response = client.bk_login.is_login({"bk_token": bk_token})
            if response["result"] is False and str(response["code"]) == USER_FORBIDDEN_CODE:
                return True, response["message"]
        except Exception as e:
            logger.debug(f"check user forbidden error: {e}")
        return False, ""



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""
import datetime
import decimal
import json
import uuid

import pytz
from django.utils.duration import duration_iso_string
from django.utils.functional import Promise
from django.utils.timezone import is_aware

from blueapps.utils.time import date_to_string, strftime_local


class DjangoJSONEncoderExtend(json.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time, decimal types and UUIDs.
    """

    def default(self, o):  # pylint: disable=too-many-return-statements
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            return strftime_local(o, fmt="%Y-%m-%d %H:%M:%S%z")
        elif isinstance(o, type(pytz.UTC)):
            return strftime_local(
                o.localize(datetime.datetime.utcnow()), fmt="%Y-%m-%d %H:%M:%S%z"
            )
        elif isinstance(o, datetime.date):
            return date_to_string(o)
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError("JSON can't represent timezone-aware times.")
            iso_format = o.isoformat()
            if o.microsecond:
                iso_format = iso_format[:12]
            return iso_format
        elif isinstance(o, datetime.timedelta):
            return duration_iso_string(o)
        elif isinstance(o, (uuid.UUID, decimal.Decimal)):
            return str(o)
        elif isinstance(o, Promise):
            return str(o)
        elif isinstance(o, bytes):
            return str(o, encoding="utf-8")
        elif isinstance(o, set):
            return list(o)

        return super(DjangoJSONEncoderExtend, self).default(o)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import json
import logging
import re

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

from .utils import check_script, html_escape, html_escape_name, url_escape

SITE_URL = settings.SITE_URL
logger = logging.getLogger("app")


class CheckXssMiddleware(MiddlewareMixin):
    def __init__(self, *args, **kwargs):
        self.__escape_param_list = []
        super(CheckXssMiddleware, self).__init__(*args, **kwargs)

    def process_view(self, request, view, args, kwargs):
        try:
            # 判断豁免权
            if getattr(view, "escape_exempt", False):
                return None

            # 获取豁免参数名
            self.__escape_param_list = (
                getattr(view, "escape_exempt_param", [])
                if getattr(view, "escape_exempt_param", False)
                else []
            )

            escape_type = None
            if getattr(view, "escape_script", False):
                escape_type = "script"
            elif getattr(view, "escape_url", False):
                escape_type = "url"
            # get参数转换
            request.GET = self.__escape_data(request.path, request.GET, escape_type)
            # post参数转换
            request.POST = self.__escape_data(request.path, request.POST, escape_type)
        except Exception as err:  # pylint: disable=broad-except
            logger.error(u"CheckXssMiddleware 转换失败！%s" % err)
        return None

    def __escape_data(self, path, query_dict, escape_type=None):
        """
        GET/POST参数转义
        """
        data_copy = query_dict.copy()
        for _get_key, _get_value_list in data_copy.lists():
            new_value_list = []
            for _get_value in _get_value_list:
                new_value = _get_value
                # json串不进行转义
                try:
                    json.loads(_get_value)
                    is_json = True
                except Exception:  # pylint: disable=broad-except
                    is_json = False
                # 转义新数据
                if not is_json:
                    try:
                        if escape_type is None:
                            use_type = self.__filter_param(path, _get_key)
                        else:
                            use_type = escape_type

                        if use_type == "url":
                            new_value = url_escape(_get_value)
                        elif use_type == "script":
                            new_value = check_script(_get_value)
                        elif use_type == "name":
                            new_value = html_escape_name(_get_value)
                        elif _get_key in self.__escape_param_list:
                            new_value = _get_value
                        else:
                            new_value = html_escape(_get_value, 1)
                    except Exception as err:  # pylint: disable=broad-except
                        logger.error(u"CheckXssMiddleware GET/POST参数 转换失败！%s" % err)
                        new_value = _get_value
                else:
                    try:
                        new_value = html_escape(_get_value, 1, True)
                    except Exception as err:  # pylint: disable=broad-except
                        logger.error(u"CheckXssMiddleware GET/POST参数 转换失败！%s" % err)
                        new_value = _get_value
                new_value_list.append(new_value)
            data_copy.setlist(_get_key, new_value_list)
        return data_copy

    def __filter_param(self, path, param):
        """
        特殊path处理
        @param path: 路径
        @param param: 参数
        @return: 'html/name/url/script/exempt'
        """
        use_name, use_url, use_script = self.__filter_path_list()
        try:
            result = "html"
            # name过滤
            for name_path, name_v in use_name.items():
                is_path = re.match(r"^%s" % name_path, path)
                if is_path and param in name_v:
                    result = "name"
                    break
            # url过滤
            if result == "html":
                for url_path, url_v in use_url.items():
                    is_path = re.match(r"^%s" % url_path, path)
                    if is_path and param in url_v:
                        result = "url"
                        break
            # script过滤
            if result == "html":
                for script_path, script_v in use_script.items():
                    is_path = re.match(r"^%s" % script_path, path)
                    if is_path and param in script_v:
                        result = "script"
                        break
        except Exception as err:  # pylint: disable=broad-except
            logger.error(u"CheckXssMiddleware 特殊path处理失败！%s" % err)
            result = "html"
        return result

    def __filter_path_list(self):
        """
        特殊path注册
        """
        use_name = {}
        use_url = {
            "%saccounts/login" % SITE_URL: ["next"],
            "%saccounts/login_page" % SITE_URL: ["req_url"],
            "%saccounts/login_success" % SITE_URL: ["req_url"],
            "%s" % SITE_URL: ["url"],
        }
        use_script = {}
        return (use_name, use_url, use_script)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""
from django.core import exceptions
from django.db import models

"""
数据库常用工具类
"""


class MultiStrSplitFieldMixin(models.Field):
    """
    多个字段，使用逗号隔开，入库list->str， 出库 str->list
    头尾都加逗号","，为了方便使用ORM的contains进行过滤且避免子集字符串的越权问题
    """

    def read_from_db(
            self, value, expression, connection
    ):  # pylint: disable=unused-argument
        """
        从db读取 -> 转为列表
        """
        if not value:
            return []
        try:
            value = value.split(",")
        except (TypeError, KeyError):
            raise exceptions.ValidationError(
                self.error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )
        result = []
        # 去除头尾逗号
        for _v in value[1:-1]:
            try:
                result.append(self.sub_type(_v))
            except ValueError:
                continue

        return result

    def write_to_db(self, value):
        """
        写入db -> 通过,拼街
        """
        if not value:
            return ""

        if isinstance(value, str):
            value = value.split(",")

        value = [str(_value) for _value in value]
        try:
            value = f",{','.join(value)},"
        except (TypeError, KeyError):
            raise exceptions.ValidationError(
                self.error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

        return value


class MultiStrSplitCharField(models.CharField, MultiStrSplitFieldMixin):
    """
    ORM Char字符串列表字段
    """

    def __init__(self, *args, **kwargs):
        self.sub_type = kwargs.get("sub_type") or str
        kwargs.pop("sub_type", "")
        super().__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection):
        """
        从DB读取 与MultiStrSplitFieldMixin一致
        """
        return super().read_from_db(value, expression, connection)

    def get_prep_value(self, value):
        """Perform preliminary non-db specific value checks and conversions."""
        return super().write_to_db(value)


class MultiStrSplitTextField(models.TextField, MultiStrSplitFieldMixin):
    """
    ORM Text字符串列表字段
    """

    def __init__(self, *args, **kwargs):
        self.sub_type = kwargs.get("sub_type") or str
        kwargs.pop("sub_type", "")
        super().__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection):
        """
        从DB读取 与MultiStrSplitFieldMixin一致
        """
        return super().read_from_db(value, expression, connection)

    def get_prep_value(self, value):
        """Perform preliminary non-db specific value checks and conversions."""
        return super().write_to_db(value)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import cryptography  # noqa
import jwt

from jwt import exceptions as jwt_exceptions

from django.conf import settings

from blueapps.utils.logger import logger
from blueapps.utils.fancy_dict import FancyDict


class JWTClient(object):
    def __init__(self, request):
        self.request = request

        self.raw_content = request.META.get(
            getattr(settings, "APIGW_JWT_KEY", "HTTP_X_BKAPI_JWT"), ""
        )
        self.error_message = ""
        self.is_valid = False

        self.payload = {}
        self.headers = {}
        self.get_jwt_info()

        self.app = self.get_app_model()
        self.user = self.get_user_model()

    def get_app_model(self):
        return FancyDict(self.payload.get("app", {}))

    def get_user_model(self):
        return FancyDict(self.payload.get("user", {}))

    def get_jwt_info(self):
        if not self.raw_content:
            self.error_message = "X_BKAPI_JWT not in http header or it is empty, please called API through API Gateway"
            return False
        try:
            self.headers = jwt.get_unverified_header(self.raw_content)
            self.payload = jwt.decode(
                self.raw_content, settings.APIGW_PUBLIC_KEY, issuer="APIGW"
            )
            self.is_valid = True
        except jwt_exceptions.InvalidKeyError:
            self.error_message = "APIGW_PUBLIC_KEY error"
        except jwt_exceptions.DecodeError:
            self.error_message = "Invalid X_BKAPI_JWT, wrong format or value"
        except jwt_exceptions.ExpiredSignatureError:
            self.error_message = "Invalid X_BKAPI_JWT, which is expired"
        except jwt_exceptions.InvalidIssuerError:
            self.error_message = "Invalid X_BKAPI_JWT, which is not from API Gateway"
        except Exception as error:
            self.error_message = error.message

        if self.error_message:
            logger.exception(
                "[jwt_client] decode jwt fail, err: %s" % self.error_message
            )

    def __str__(self):
        return "<{headers}, {payload}>".format(
            headers=self.headers, payload=self.payload
        )



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import os

from blueapps.conf import get_settings_from_module, get_config_default_module

default = get_config_default_module()

locals().update(get_settings_from_module(default))

INSTALLED_APPS = locals()["INSTALLED_APPS"]
REMOTE_STATIC_URL = locals()["REMOTE_STATIC_URL"]

if not REMOTE_STATIC_URL.endswith("/"):
    REMOTE_STATIC_URL = "%s/" % REMOTE_STATIC_URL

# saas访问统计js 路径
REMOTE_ANALYSIS_URL = "%sanalysis.js" % REMOTE_STATIC_URL

# paas提供的前端api.js 路径
REMOTE_API_URL = "%sbk_api/api.js" % REMOTE_STATIC_URL

# 从 apigw jwt 中获取 app_code 的 键
APIGW_APP_CODE_KEY = "app_code"

# 从 apigw jwt 中获取 username 的 键
APIGW_USER_USERNAME_KEY = "username"

# sentry support

SENTRY_DSN = os.environ.get("SENTRY_DSN")
if SENTRY_DSN:
    INSTALLED_APPS += ("raven.contrib.django.raven_compat",)
    RAVEN_CONFIG = {
        "dsn": SENTRY_DSN,
    }

# 非open环境使用页面的语言切换按钮来控制语言
IS_DISPLAY_LANGUAGE_CHANGE = "block"



import os
import shutil
from django.conf import settings
from django.core.management.base import BaseCommand
from blueapps.apigw_sync.apigw_sync_settings import apigw_settings


class Command(BaseCommand):
    help = 'Generate api resources'

    def add_arguments(self, parser):
        parser.add_argument('--target_dir', type=str, help='Target path for the output file')

    def handle(self, *args, **options):
        # 获取模板文件输出根目录相对路径
        target_dir = options['target_dir'] if options['target_dir'] \
            else apigw_settings.GENERATE_API_RESOURCES_TARGET_DIR
        # 获取模板文件名
        resources_file_name = apigw_settings.API_RESOURCES_FILE_NAME
        # 获取模板文件
        curent_dir = os.path.dirname(os.path.abspath(__file__))
        source_file = os.path.join(curent_dir, 'data', resources_file_name)
        # 获取模板文件输出绝对路径
        target_dir = os.path.join(settings.BASE_DIR, target_dir.strip('/'))
        os.makedirs(target_dir, exist_ok=True)

        target_file = os.path.join(target_dir, resources_file_name)
        if os.path.exists(target_file):
            self.stdout.write(
                self.style.ERROR(f'generate api resources template file to {target_file} failed: file already exists'))
            return

        shutil.copy(source_file, target_file)
        self.stdout.write(self.style.SUCCESS(f'Successfully generate api resources template file to {target_file}'))



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import re

from html.parser import HTMLParser


"""
Python 富文本XSS过滤类
@package XssHtml
@version 0.1
@link http://phith0n.github.io/python-xss-filter
@since 20150407
@copyright (c) Phithon All Rights Reserved
Based on native Python module HTMLParser purifier of HTML, To Clear all javascript in html
You can use it in all python web framework
Written by Phithon <root@leavesongs.com> in 2015 and placed in the public domain.
phithon <root@leavesongs.com> 编写于20150407
From: XDSEC <www.xdsec.org> & 离别歌 <www.leavesongs.com>
GitHub Pages: https://github.com/phith0n/python-xss-filter
Usage:
    parser = XssHtml()
    parser.feed('<html code>')
    parser.close()
    html = parser.get_html()
    print html
Requirements
Python 2.6+ or 3.2+
Cannot defense xss in browser which is belowed IE7
浏览器版本：IE7+ 或其他浏览器，无法防御IE6及以下版本浏览器中的XSS
"""


class XssHtml(HTMLParser):
    allow_tags = [
        "a",
        "img",
        "br",
        "strong",
        "b",
        "code",
        "pre",
        "p",
        "div",
        "em",
        "span",
        "h1",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "blockquote",
        "ul",
        "ol",
        "tr",
        "th",
        "td",
        "hr",
        "li",
        "u",
        "embed",
        "s",
        "table",
        "thead",
        "tbody",
        "caption",
        "small",
        "q",
        "sup",
        "sub",
    ]
    common_attrs = ["id", "style", "class", "name"]
    nonend_tags = ["img", "hr", "br", "embed"]
    tags_own_attrs = {
        "img": ["src", "width", "height", "alt", "align"],
        "a": ["href", "target", "rel", "title"],
        "embed": [
            "src",
            "width",
            "height",
            "type",
            "allowfullscreen",
            "loop",
            "play",
            "wmode",
            "menu",
        ],
        "table": ["border", "cellpadding", "cellspacing"],
    }

    def __init__(self, allows=None):
        HTMLParser.__init__(self)
        if allows is None:
            allows = []
        self.allow_tags = allows if allows else self.allow_tags
        self.result = []
        self.start = []
        self.data = []

    def get_html(self):
        """
        Get the safe html code
        """
        for i in range(0, len(self.result)):
            tmp = self.result[i].rstrip("\n")
            tmp = tmp.lstrip("\n")
            if tmp:
                self.data.append(tmp)
        return "".join(self.data)

    def handle_startendtag(self, tag, attrs):
        self.handle_starttag(tag, attrs)

    def handle_starttag(self, tag, attrs):
        if tag not in self.allow_tags:
            return
        end_diagonal = " /" if tag in self.nonend_tags else ""
        if not end_diagonal:
            self.start.append(tag)
        attdict = {}
        for attr in attrs:
            attdict[attr[0]] = attr[1]

        attdict = self.__wash_attr(attdict, tag)
        if hasattr(self, "node_%s" % tag):
            attdict = getattr(self, "node_%s" % tag)(attdict)
        else:
            attdict = self.node_default(attdict)

        attrs = []
        for (key, value) in attdict.items():
            attrs.append('{}="{}"'.format(key, self.__htmlspecialchars(value)))
        attrs = (" " + " ".join(attrs)) if attrs else ""
        self.result.append("<" + tag + attrs + end_diagonal + ">")

    def handle_endtag(self, tag):
        if self.start and tag == self.start[len(self.start) - 1]:
            self.result.append("</" + tag + ">")
            self.start.pop()

    def handle_data(self, data):
        self.result.append(self.__htmlspecialchars(data))

    def handle_entityref(self, name):
        if name.isalpha():
            self.result.append("&%s;" % name)

    def handle_charref(self, name):
        if name.isdigit():
            self.result.append("&#%s;" % name)

    def node_default(self, attrs):
        attrs = self.__common_attr(attrs)
        return attrs

    def node_a(self, attrs):
        attrs = self.__common_attr(attrs)
        attrs = self.__get_link(attrs, "href")
        attrs = self.__set_attr_default(attrs, "target", "_blank")
        attrs = self.__limit_attr(attrs, {"target": ["_blank", "_self"]})
        return attrs

    def node_embed(self, attrs):
        attrs = self.__common_attr(attrs)
        attrs = self.__get_link(attrs, "src")
        attrs = self.__limit_attr(
            attrs,
            {
                "type": ["application/x-shockwave-flash"],
                "wmode": ["transparent", "window", "opaque"],
                "play": ["true", "false"],
                "loop": ["true", "false"],
                "menu": ["true", "false"],
                "allowfullscreen": ["true", "false"],
            },
        )
        attrs["allowscriptaccess"] = "never"
        attrs["allownetworking"] = "none"
        return attrs

    def __true_url(self, url):
        prog = re.compile(r"^(http|https|ftp)://.+", re.I | re.S)
        if prog.match(url):
            return url
        else:
            return "http://%s" % url

    def __true_style(self, style):
        if style:
            style = re.sub(r"(\\|&#|/\*|\*/)", "_", style)
            style = re.sub(r"e.*x.*p.*r.*e.*s.*s.*i.*o.*n", "_", style)
        return style

    def __get_style(self, attrs):
        if "style" in attrs:
            attrs["style"] = self.__true_style(attrs.get("style"))
        return attrs

    def __get_link(self, attrs, name):
        if name in attrs:
            attrs[name] = self.__true_url(attrs[name])
        return attrs

    def __wash_attr(self, attrs, tag):
        if tag in self.tags_own_attrs:
            other = self.tags_own_attrs.get(tag)
        else:
            other = []
        if attrs:
            for (key, _) in list(attrs.items()):
                if key not in self.common_attrs + other:
                    del attrs[key]
        return attrs

    def __common_attr(self, attrs):
        attrs = self.__get_style(attrs)
        return attrs

    def __set_attr_default(self, attrs, name, default=""):
        if name not in attrs:
            attrs[name] = default
        return attrs

    def __limit_attr(self, attrs, limit=None):
        if limit is None:
            limit = {}
        for (key, value) in limit.items():
            if key in attrs and attrs[key] not in value:
                del attrs[key]
        return attrs

    def __htmlspecialchars(self, html):
        return (
            html.replace("<", "&lt;")
            .replace(">", "&gt;")
            .replace('"', "&quot;")
            .replace("'", "&#039;")
        )


if "__main__" == __name__:
    parser = XssHtml()
    parser.feed(
        """<p><img src=1 onerror=alert(/xss/)></p><div class="left">
        <a href='javascript:prompt(1)'><br />hehe</a></div>
        <p id="test" onmouseover="alert(1)">&gt;M<svg>
        <a href="https://www.baidu.com" target="self">MM</a></p>
        <embed src='javascript:alert(/hehe/)' allowscriptaccess=always />"""
    )
    parser.close()
    print(parser.get_html())



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""
from rest_framework import filters
from rest_framework.viewsets import GenericViewSet, ModelViewSet

from blueapps.contrib.drf.utils.serializers import GeneralSerializer
from blueapps.contrib.drf.utils.viewset_mixin import ApiMixin, ValidationMixin


class APIViewSet(ApiMixin, ValidationMixin, GenericViewSet):
    """封装 APIViewSet"""


class Meta:
    """Empty object"""

    pass


class GeneralModelViewSet(ApiMixin, ValidationMixin, ModelViewSet):
    """
    DRF 自定义modelViewSet
    """

    model = None
    filter_backends = (
        filters.OrderingFilter,
        filters.SearchFilter,
    )
    serializer_meta = type("Meta", (Meta,), {"model": None, "ref_name": None})

    def __init__(self, *args, **kwargs):
        super(GeneralModelViewSet, self).__init__(**kwargs)
        self.filter_fields = [f.name for f in self.model._meta.get_fields()]
        self.view_set_name = self.get_view_object_name(*args, **kwargs)

    def get_view_name(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Return the view name, as used in OPTIONS responses and in the
        browsable API.
        """
        return self.model._meta.db_table  # pylint: disable=protected-access

    def get_view_description(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Return some descriptive text for the view, as used in OPTIONS responses
        and in the browsable API.
        """
        return self.model._meta.verbose_name  # pylint: disable=protected-access

    def get_view_module(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        获取视图设置的model
        """
        return getattr(self.model._meta, "module", None)  # pylint: disable=protected-access

    def get_view_object_name(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        获取视图设置的model名称
        """
        return getattr(self.model._meta, "object_name", None)  # pylint: disable=protected-access

    def get_queryset(self):
        """
        重写视图类get_queryset方法
        """
        return self.model.objects.all()

    def get_serializer_class(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        获取视图类序列化器
        """
        self.serializer_meta.model = self.model
        self.serializer_meta.fields = "__all__"
        if isinstance(self.serializer_class, GeneralSerializer) or self.serializer_class is None:
            self.serializer_meta.ref_name = self.model.__name__
            return type(
                "GeneralSerializer{}".format(self.model.__name__), (GeneralSerializer,), {"Meta": self.serializer_meta},
            )

        return self.serializer_class



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import json
import logging
import traceback

from django.conf import settings
from django.http import Http404, JsonResponse
from django.utils.deprecation import MiddlewareMixin
from django.utils.translation import gettext_lazy as _

from blueapps.core.exceptions.base import BlueException

try:
    from raven.contrib.django.raven_compat.models import sentry_exception_handler
# 兼容未有安装sentry的情况
except ImportError:
    sentry_exception_handler = None

logger = logging.getLogger("blueapps")


class AppExceptionMiddleware(MiddlewareMixin):
    def process_exception(self, request, exception):
        """
        app后台错误统一处理
        """

        self.exception = exception
        self.request = request

        # 用户自我感知的异常抛出
        if isinstance(exception, BlueException):
            logger.log(
                exception.LOG_LEVEL,
                (
                    u"""捕获主动抛出异常, 具体异常堆栈->[%s] status_code->[%s] & """
                    u"""client_message->[%s] & args->[%s] """
                )
                % (
                    traceback.format_exc(),
                    exception.ERROR_CODE,
                    exception.message,
                    exception.args,
                ),
            )

            response = JsonResponse(exception.response_data())

            response.status_code = exception.STATUS_CODE
            return response

        # 用户未主动捕获的异常
        logger.error(
            u"""捕获未处理异常,异常具体堆栈->[%s], 请求URL->[%s], """
            u"""请求方法->[%s] 请求参数->[%s]"""
            % (
                traceback.format_exc(),
                request.path,
                request.method,
                json.dumps(getattr(request, request.method, None)),
            )
        )

        # 对于check开头函数进行遍历调用，如有满足条件的函数，则不屏蔽异常
        check_funtions = self.get_check_functions()
        for check_function in check_funtions:
            if check_function():
                return None

        response = JsonResponse(
            {
                "result": False,
                "code": "50000",
                "message": _(u"系统异常,请联系管理员处理"),
                "data": None,
            }
        )
        response.status_code = 500

        # notify sentry
        if sentry_exception_handler is not None:
            sentry_exception_handler(request=request)

        return response

    def get_check_functions(self):
        """获取需要判断的函数列表"""
        return [
            getattr(self, func)
            for func in dir(self)
            if func.startswith("check") and callable(getattr(self, func))
        ]

    def check_is_debug(self):
        """判断是否是开发模式"""
        return settings.DEBUG

    def check_is_http404(self):
        """判断是否基于Http404异常"""
        return isinstance(self.exception, Http404)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

try:
    from collections import Callable
except ImportError:
    from collections.abc import Callable

from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string

from blueapps.conf import settings
from blueapps.core.exceptions import AccessForbidden, MethodError
from blueapps.utils.request_provider import get_request

"""
全平台 esb-sdk 封装，依赖于 esb-sdk 包，但不依赖 sdk 的版本。
sdk 中有封装好 cc.get_app_by_user 方法时，可直接按以前 sdk 的习惯调用

from blueapps.utils import client
client.cc.get_app_by_user()

from blueapps.utils import backend_client
b_client = backend_client(access_token="SfgcGlBHmPWttwlGd7nOLAbOP3TAOG")
b_client.cc.get_app_by_user()

当前版本 sdk 中未封装好，但 api 已经有 get_app_by_user 的时候。需要指定请求方法
client.cc.get_app_by_user.get()
"""

__all__ = [
    "client",
    "backend_client",
    "get_client_by_user",
    "get_client_by_request",
    "CustomComponentAPI",
]


# esb api的url path前缀
def get_api_prefix():
    platform_api_prefix_map = {
        # 内部版
        "ieod": "/component/compapi/",
        # tencet
        "tencent": "/c/ieg/compapi",
        # open
        "open": "/api/c/compapi/",
    }
    return platform_api_prefix_map[settings.RUN_VER]


ESB_API_PREFIX = get_api_prefix()

try:
    ESB_SDK_NAME = settings.ESB_SDK_NAME
    if not ESB_SDK_NAME:
        raise AttributeError
except AttributeError:
    ESB_SDK_NAME = "blueking.component.{platform}".format(platform=settings.RUN_VER)


class SDKClient(object):
    sdk_package = None

    @property
    def __version__(self):
        return self.sdk_package.__version__

    @property
    def __backend__(self):
        return self.sdk_package.__name__

    def __new__(cls, **kwargs):
        if cls.sdk_package is None:
            try:
                cls.sdk_package = __import__(ESB_SDK_NAME, fromlist=["shortcuts"])
            except ImportError as err:
                raise ImportError("%s is not installed: %s" % (ESB_SDK_NAME, err))
        return super(SDKClient, cls).__new__(cls)

    def __init__(self, **kwargs):
        self.mod_name = ""
        self.sdk_mod = None
        for ignored_field in ["app_code", "app_secret"]:
            if ignored_field in kwargs:
                kwargs.pop(ignored_field)
        self.common_args = kwargs

    def __getattr__(self, item):
        if not self.mod_name:
            ret = SDKClient(**self.common_args)
            ret.mod_name = item
            ret.setup_modules()
            if isinstance(ret.sdk_mod, Callable):
                return ret.sdk_mod
            return ret
        else:
            # 真实sdk调用入口
            ret = getattr(self.sdk_mod, item, None)
            if ret is None:
                ret = ComponentAPICollection(self).add_api(item)
        if not isinstance(ret, Callable):
            ret = self
        return ret

    def setup_modules(self):
        self.sdk_mod = getattr(self.sdk_client, self.mod_name, None)
        if self.sdk_mod is None:
            self.sdk_mod = ComponentAPICollection(self)

    @property
    def sdk_client(self):
        try:
            request = get_request()
            # 调用sdk方法获取sdk client
            return self.load_sdk_class("shortcuts", "get_client_by_request")(request)
        except Exception:  # pylint: disable=broad-except
            if settings.RUN_MODE != "DEVELOP":
                if self.common_args:
                    return self.load_sdk_class("client", "ComponentClient")(
                        app_code=settings.APP_CODE, app_secret=settings.SECRET_KEY, common_args=self.common_args,
                    )
                else:
                    raise AccessForbidden("sdk can only be called through the Web request")
            else:
                # develop mode
                # 根据RUN_VER获得get_component_client_common_args函数
                get_component_client_common_args = import_string(
                    "blueapps.utils.sites.{platform}."
                    "get_component_client_common_args".format(platform=settings.RUN_VER)
                )
                return self.load_sdk_class("client", "ComponentClient")(
                    app_code=settings.APP_CODE,
                    app_secret=settings.SECRET_KEY,
                    common_args=get_component_client_common_args(),
                )

    def load_sdk_class(self, mod, attr_or_class):
        dotted_path = "{}.{}.{}".format(self.__backend__, mod, attr_or_class)
        return import_string(dotted_path)

    def patch_sdk_component_api_class(self):
        def patch_get_item(self, item):
            if item.startswith("__"):
                # make client can be pickled
                raise AttributeError()

            method = item.upper()
            if method not in self.allowed_methods:
                raise MethodError("esb api does not support method: %s" % method)
            self.method = method
            return self

        api_cls = self.load_sdk_class("base", "ComponentAPI")
        setattr(api_cls, "allowed_methods", CustomComponentAPI.allowed_methods)
        setattr(api_cls, "__getattr__", patch_get_item)


class ComponentAPICollection(object):
    mod_map = dict()

    def __new__(cls, sdk_client, *args, **kwargs):
        if sdk_client.mod_name not in cls.mod_map:
            cls.mod_map[sdk_client.mod_name] = super(ComponentAPICollection, cls).__new__(cls)
        return cls.mod_map[sdk_client.mod_name]

    def __init__(self, sdk_client):
        self.client = sdk_client

    def add_api(self, action):
        custom_api = CustomComponentAPI(self, action)
        setattr(self, action, custom_api)
        return custom_api

    def __getattr__(self, item):
        api = self.add_api(item)
        return api


class CustomComponentAPI(object):
    allowed_methods = ["GET", "POST"]

    def __init__(self, collection, action):
        self.collection = collection
        self.action = action

    def __getattr__(self, method):
        method = method.upper()
        if method not in self.allowed_methods:
            raise MethodError("esb api does not support method: %s" % method)
        api_cls = self.collection.client.load_sdk_class("base", "ComponentAPI")
        return api_cls(
            client=SDKClient(**self.collection.client.common_args),
            method=method,
            path="{api_prefix}{collection}/{action}/".format(
                api_prefix=ESB_API_PREFIX, collection=self.collection.client.mod_name, action=self.action,
            ),
            description="custom api(%s)" % self.action,
        )

    def __call__(self, *args, **kwargs):
        raise NotImplementedError("custom api `%s` must specify the request method" % self.action)


try:
    client = SDKClient()
    backend_client = SDKClient  # pylint: disable=invalid-name
    client.patch_sdk_component_api_class()
except ImportError as err:
    from blueapps.contrib.dummy.dummy_client import DummyClient

    client = DummyClient(err)
    backend_client = DummyClient  # pylint: disable=invalid-name


def get_client_by_user(user_or_username):
    user_model = get_user_model()
    if isinstance(user_or_username, user_model):
        username = user_or_username.username
    else:
        username = user_or_username
    get_client_by_user = import_string(".".join([ESB_SDK_NAME, "shortcuts", "get_client_by_user"]))
    return get_client_by_user(username)


def get_client_by_request(request=None):
    return client



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from __future__ import absolute_import, unicode_literals

import os

from celery import Celery, platforms
from django.conf import settings

# http://docs.celeryproject.org/en/latest/userguide/daemonizing.html#running-the-worker-with-superuser-privileges-root
# for root start celery

platforms.C_FORCE_ROOT = True

# set the default Django settings module for the 'celery' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "settings")

app = Celery("proj")

# Using a string here means the worker don't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object("django.conf:settings")

# Load task modules from all registered Django app configs.
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)

@app.task(bind=True)
def debug_task(self):
    print("Request: {!r}".format(self.request))



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()
version = __import__("blueapps").__version__

setup(
    name="blueapps",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,
    description="development framework for blueking",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # The project's main homepage.
    url="https://github.com/TencentBlueKing/blueapps",
    # Author details
    author="blueking",
    author_email="blueking@tencent.com",
    include_package_data=True,
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),
    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        "Django>=2.2.6,<5.0.0",
        "mysqlclient>=1.4.4,<3",
        "bkoauth>=0.0.12",
        "MarkupSafe>=1.1.1,<3.0.0",
        "Mako>=1.0.6,<2.0",
        "requests>=2.22.0,<3.0.0",
        "python-json-logger>=0.1.7",
        "whitenoise>=3.3.0,<7",
        "Werkzeug>=1.0.0,<2.0.0",
        # jwt
        "pyCryptodome>=3.9.7",
        "PyJWT>=1.6.1,<3.0",
        "cryptography>=2.7",
    ],
    extras_require={
        "opentelemetry": [
            "opentelemetry-api>=1.6.2,<2.0.0",
            "opentelemetry-sdk>=1.6.2,<2.0.0",
            "opentelemetry-exporter-otlp>=1.6.2,<2.0.0",
            "opentelemetry-exporter-jaeger>=1.6.2,<2.0.0",
            "opentelemetry-exporter-jaeger-proto-grpc>=1.6.2,<2.0.0",
            "opentelemetry-exporter-jaeger-thrift>=1.6.2,<2.0.0",
            "opentelemetry-instrumentation>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-celery>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-django>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-dbapi>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-redis>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-logging>=0.25b2,<1.0.0",
            "opentelemetry-instrumentation-requests>=0.25b2,<1.0.0",
            "prometheus-client>=0.9.0,<1.0.0",
            "django-prometheus>=2.1.0,<3.0.0",
        ],
        "bkcrypto": ["bk-crypto-python-sdk==1.1.1"],
        "bk-notice": ["bk-notice-sdk==1.1.1"],
        "apigw-manager": ["apigw-manager>=1.1.5,<3.0.0"],
    },
    zip_safe=False,
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={"console_scripts": ["bk-admin=blueapps.contrib.bk_commands:bk_admin"]},
)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from django.core.management import BaseCommand
from django.db import transaction
from django_celery_beat.models import (
    IntervalSchedule,
    CrontabSchedule,
    PeriodicTasks,
    PeriodicTask,
)

from blueapps.contrib.bk_commands.management.handlers.migrate_from_djcelery_handler import (
    execute,
    DjIntervalSchedule,
    DjCrontabSchedule,
    DjPeriodicTask,
    DjPeriodicTasks,
)

logger = logging.getLogger("blueapps")


class Command(BaseCommand):
    ALL_MIGRATED_DB_TABLE = (IntervalSchedule, CrontabSchedule, PeriodicTasks, PeriodicTask)
    NEW_NO_TIMEZONE_DB_TABLE = (IntervalSchedule, PeriodicTasks, PeriodicTask)
    OLD_DB_TABLE = (DjIntervalSchedule, DjPeriodicTasks, DjPeriodicTask)

    def add_arguments(self, parser):
        parser.add_argument("-tz", help="指定旧版djcelery运行的时区")

    @transaction.atomic
    def handle(self, *args, **options):
        tz = "UTC"  # pylint: disable=invalid-name
        if options["tz"]:
            tz = options["tz"]  # pylint: disable=invalid-name
        # 检查目标数据库是否有数据
        if not self.check_db_has_data():
            return
        # 迁移带时区的表
        execute(CrontabSchedule, DjCrontabSchedule, tz)
        # 迁移不带时区的表
        for new_table, old_table in zip(self.NEW_NO_TIMEZONE_DB_TABLE, self.OLD_DB_TABLE):
            execute(new_table, old_table)

    def check_db_has_data(self):
        for db in self.ALL_MIGRATED_DB_TABLE:
            if db.objects.exists():
                logger.warning("The target database {} already has data and cannot be migrated"
                               .format(db._meta.model_name))
                return False
        return True



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from typing import Collection

from django import VERSION as django_version
from django.conf import settings
from opentelemetry.instrumentation.django.package import _instruments
from opentelemetry.instrumentation.instrumentor import BaseInstrumentor

from .middlewares import SaaSMetricsBeforeMiddleware, SaaSMetricsAfterMiddleware

DJANGO_2_0 = django_version >= (2, 0)


def _get_django_middleware_setting() -> str:
    # In Django versions 1.x, setting MIDDLEWARE_CLASSES can be used as a legacy
    # alternative to MIDDLEWARE. This is the case when `settings.MIDDLEWARE` has
    # its default value (`None`).
    if not DJANGO_2_0 and getattr(settings, "MIDDLEWARE", None) is None:
        return "MIDDLEWARE_CLASSES"
    return "MIDDLEWARE"


class SaaSMetricsInstrumentor(BaseInstrumentor):

    _before_middleware = ".".join(
        [
            SaaSMetricsBeforeMiddleware.__module__,
            SaaSMetricsBeforeMiddleware.__qualname__,
        ]
    )
    _after_middleware = ".".join(
        [SaaSMetricsAfterMiddleware.__module__, SaaSMetricsAfterMiddleware.__qualname__]
    )

    def instrumentation_dependencies(self) -> Collection[str]:
        return _instruments

    def _instrument(self, **kwargs):
        # This can not be solved, but is an inherent problem of this approach:
        # the order of middleware entries matters, and here you have no control
        # on that:
        # https://docs.djangoproject.com/en/3.0/topics/http/middleware/#activating-middleware
        # https://docs.djangoproject.com/en/3.0/ref/middleware/#middleware-ordering

        _middleware_setting = _get_django_middleware_setting()
        settings_middleware = getattr(settings, _middleware_setting, [])

        # Django allows to specify middlewares as a tuple, so we convert this tuple to a
        # list, otherwise we wouldn't be able to call append/remove
        if isinstance(settings_middleware, tuple):
            settings_middleware = list(settings_middleware)

        settings_middleware.insert(0, self._before_middleware)
        settings_middleware.append(self._after_middleware)
        setattr(settings, _middleware_setting, settings_middleware)

    def _uninstrument(self, **kwargs):
        _middleware_setting = _get_django_middleware_setting()
        settings_middleware = getattr(settings, _middleware_setting, None)

        if settings_middleware is None or (
            self._before_middleware not in settings_middleware
            and self._after_middleware not in settings_middleware
        ):
            return

        settings_middleware.remove(self._before_middleware)
        settings_middleware.remove(self._after_middleware)
        setattr(settings, _middleware_setting, settings_middleware)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from django.conf import settings
from django.contrib import auth
from django.core.cache import caches
from django.utils.deprecation import MiddlewareMixin

from blueapps.account.components.bk_ticket.forms import AuthenticationForm
from blueapps.account.conf import ConfFixture
from blueapps.account.handlers.response import ResponseHandler

logger = logging.getLogger("component")
cache = caches["login_db"]


class LoginRequiredMiddleware(MiddlewareMixin):
    def process_view(self, request, view, args, kwargs):
        """
        可通过登录认证的方式，仅有两种
        1. 带有 login_exemp 标识的 view 函数
        2. 需要进行验证逻辑：
        # 先获得 cookie 并校验，失败进入 401 页面进行登录，其他情况继续执行
        # 在 session 和 cookie 中的 bk_ticket 是否相同且用户有效直接 None 放行页面
        # 不满足上面的条件，则需要对 bk_ticket 进行校验，并返回 user ，
        如果 user 对象存在，进行将 bk_ticket 写入 session
        # 其他情况返回 401 页面进行登录
        """
        if hasattr(request, "is_wechat") and request.is_wechat():
            return None

        if hasattr(request, "is_rio") and request.is_rio():
            return None

        if hasattr(request, "is_bk_jwt") and request.is_bk_jwt():
            return None

        # 登录豁免
        if getattr(view, "login_exempt", False):
            return None

        user = self.authenticate(request)
        if user:
            return None

        # 验证不通过，需要跳转至统一登录平台
        handler = ResponseHandler(ConfFixture, settings)
        return handler.build_401_response(request)

    def process_response(self, request, response):
        return response

    def authenticate(self, request):
        form = AuthenticationForm(request.COOKIES)
        if not form.is_valid():
            return None

        bk_ticket = form.cleaned_data["bk_ticket"]
        session_key = request.session.session_key
        # 确认 cookie 中的 ticket 和 cache 中的是否一致
        if session_key:
            cache_session = cache.get(session_key)
            is_match = cache_session and bk_ticket == cache_session.get("bk_ticket")
            if is_match and request.user.is_authenticated:
                return request.user
        user = auth.authenticate(request=request, bk_ticket=bk_ticket)
        if user is not None and user.username != request.user.username:
            auth.login(request, user)
        if user is not None and request.user.is_authenticated:
            # 登录成功，重新调用自身函数，即可退出
            cache.set(
                session_key, {"bk_ticket": bk_ticket}, settings.LOGIN_CACHE_EXPIRED
            )
            return self.authenticate(request)
        return user



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from .pxfilter import XssHtml


"""
蓝鲸平台提供的公用方法

#===============================================================================
# 1.页面输入内容转义（防止xss攻击）
from common.utils import html_escape, url_escape

 # 转义html内容
 html_content = html_escape(input_content)

 # 转义url内容
 url_content = url_escape(input_content)
#===============================================================================
"""


def html_escape(str_escape, fromtype=0, is_json=False):
    """
    字符串转义为html代码
    @param str_escape: 需要解析的html代码
    @param fromtype: 来源，0：views函数，1：middleware
    @param is_json: 是否为json串
    """
    try:
        result_str = escape_new(str_escape, fromtype, is_json)
        return result_str
    except Exception:  # pylint: disable=broad-except
        return str_escape


def url_escape(url_escape):
    """
    转义url中的特殊字符
    @param str_escape: 需要解析的url
    """
    try:
        result_str = escape_url(url_escape)
        return result_str
    except Exception:  # pylint: disable=broad-except
        return url_escape


def html_escape_name(str_escape):
    """
    字符串转义为html代码
    @param str_escape: 需要解析的html代码
    """
    try:
        result_str = escape_name(str_escape)
        return result_str
    except Exception:  # pylint: disable=broad-except
        return str_escape


def escape_url(input_str):
    input_str = input_str.replace("<", "")
    input_str = input_str.replace(">", "")
    input_str = input_str.replace(" ", "")
    input_str = input_str.replace('"', "")
    input_str = input_str.replace("'", "")
    return input_str


def escape_name(input_str):
    """Replace special characters "&", "<" and ">" to HTML-safe sequences.
    If the optional flag quote is true, the quotation mark character (")
    is also translated.
    rewrite the cgi method
    """
    input_str = input_str.replace("&", "")  # Must be done first!
    input_str = input_str.replace("<", "")
    input_str = input_str.replace(">", "")
    input_str = input_str.replace(" ", "")
    input_str = input_str.replace('"', "")
    input_str = input_str.replace("'", "")
    return input_str


def check_script(str_escape):
    """
    防止js脚本注入
    @param str_escape: 要检测的字符串
    @param fromtype: 0：views，1：middleware
    """
    try:
        parser = XssHtml()
        parser.feed(str_escape)
        parser.close()
        return parser.get_html()
    except Exception:  # pylint: disable=broad-except
        return str_escape


def escape_new(input_str, fromtype, is_json):
    """Replace special characters "&", "<" and ">" to HTML-safe sequences.
    If the optional flag quote is true, the quotation mark character (")
    is also translated.
    rewrite the cgi method
    @param fromtype: 来源，0：views函数，1：middleware（对&做转换），默认是0
    @param is_json: 是否为json串（True/False
    """
    # &转换
    if fromtype == 1 and not is_json:
        input_str = input_str.replace("&", "&amp;")
    # <>转换
    input_str = input_str.replace("<", "&lt;")
    input_str = input_str.replace(">", "&gt;")
    # 单双引号转换
    if not is_json:
        input_str = input_str.replace(" ", "&nbsp;")
        input_str = input_str.replace('"', "&quot;")
        input_str = input_str.replace("'", "&#39;")
    return input_str



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import os

from blueapps.conf import get_settings_from_module, get_config_default_module, get_config_module
from blueapps.patch.log import get_paas_v2_logging_config_dict

get_env_or_raise = get_config_module().get_env_or_raise
default = get_config_default_module()

locals().update(get_settings_from_module(default))

BK_URL = locals()["BK_URL"]
MIDDLEWARE = locals()["MIDDLEWARE"]
INSTALLED_APPS = locals()["INSTALLED_APPS"]
STATIC_URL = locals()["STATIC_URL"]
RUN_VER = locals()["RUN_VER"]

# ESB component
if not locals().get("ESB_SDK_NAME"):
    ESB_SDK_NAME = "blueking.component"

# 从 apigw jwt 中获取 app_code 的 键
APIGW_APP_CODE_KEY = "bk_app_code"

# 从 apigw jwt 中获取 username 的 键
APIGW_USER_USERNAME_KEY = "bk_username"

# PAASV2对外版不需要bkoauth,DISABLED_APPS加入bkoauth
INSTALLED_APPS = (
    INSTALLED_APPS[0 : INSTALLED_APPS.index("bkoauth")]
    + INSTALLED_APPS[INSTALLED_APPS.index("bkoauth") + 1 :]
)

# 请求官方 API 默认版本号，可选值为："v2" 或 ""；其中，"v2"表示规范化API，
# ""表示未规范化API.如果外面设置了该值则使用设置值,否则默认使用v2
DEFAULT_BK_API_VER = locals().get("DEFAULT_BK_API_VER", "v2")

# open环境使用cookie中的blueking_language来控制语言
LANGUAGE_SESSION_KEY = "blueking_language"
LANGUAGE_COOKIE_NAME = "blueking_language"
IS_DISPLAY_LANGUAGE_CHANGE = "none"

# CSRF Config
CSRF_COOKIE_NAME = "csrftoken"

IS_OPEN_V3 = os.getenv("BKPAAS_MAJOR_VERSION", False)


# 判断open环境对应的PaaS版本
def is_open_saas_v2():
    return RUN_VER == "open" and not IS_OPEN_V3


if is_open_saas_v2():  # V2
    # 蓝鲸PASS平台URL
    BK_PAAS_HOST = os.getenv("BK_PAAS_HOST", BK_URL).rstrip("/")

    # 用于 用户认证、用户信息获取 的蓝鲸主机
    BK_PAAS_INNER_HOST = os.getenv("BK_PAAS_INNER_HOST", BK_PAAS_HOST).rstrip("/")

    # 兼容component的APP_ID,APP_TOKEN
    APP_CODE = APP_ID = get_env_or_raise("APP_ID")
    SECRET_KEY = APP_TOKEN = get_env_or_raise("APP_TOKEN")

    # IS_LOCAL,V2和V3的判断方式不同,V2用BK_ENV
    IS_LOCAL = not os.getenv("BK_ENV", False)

    # 日志
    BK_LOG_DIR = os.getenv("BK_LOG_DIR", "/data/apps/logs/")
    LOGGING = get_paas_v2_logging_config_dict(
        is_local=IS_LOCAL,
        bk_log_dir=BK_LOG_DIR,
        log_level=locals().get("LOG_LEVEL", "INFO"),
    )

    # PAASV2对外版不需要whitenoise,MIDDLEWARE中去除'whitenoise.middleware.WhiteNoiseMiddleware'
    MIDDLEWARE = (
        MIDDLEWARE[0 : MIDDLEWARE.index("whitenoise.middleware.WhiteNoiseMiddleware")]
        + MIDDLEWARE[
            MIDDLEWARE.index("whitenoise.middleware.WhiteNoiseMiddleware") + 1 :
        ]
    )

    # BROKER_URL
    if "BK_BROKER_URL" in os.environ:
        BROKER_URL = os.getenv("BK_BROKER_URL")

    # SITE_URL,STATIC_URL,,FORCE_SCRIPT_NAME
    # 测试环境
    if os.getenv("BK_ENV") == "testing":
        BK_URL = os.environ.get("BK_URL", "%s/console/" % BK_PAAS_HOST)
        SITE_URL = os.environ.get("BK_SITE_URL", "/t/%s/" % APP_CODE)
        STATIC_URL = "%sstatic/" % SITE_URL
    # 正式环境
    if os.getenv("BK_ENV") == "production":
        BK_URL = os.environ.get("BK_URL", "%s/console/" % BK_PAAS_HOST)
        SITE_URL = os.environ.get("BK_SITE_URL", "/o/%s/" % APP_CODE)
        STATIC_URL = "%sstatic/" % SITE_URL

    # STATIC_ROOT,静态文件收集文件夹,由于企业版需要用户手动收集,此处设为空,
    # 同时需要设置STATICFILES_DIRS不改变
    STATIC_ROOT = None

    # saas访问统计js 路径
    REMOTE_ANALYSIS_URL = "%s/console/static/js/analysis.min.js" % BK_PAAS_HOST

    # paas提供的前端api.js 路径
    REMOTE_API_URL = "%s/console/static/bk_api/api.js" % BK_PAAS_HOST

    # 蓝鲸开发者页面
    BK_DEV_URL = "%s/app/list/" % BK_PAAS_HOST

    # 蓝鲸登陆页面
    BK_LOGIN_URL = BK_PAAS_HOST + "/login"
    BK_LOGIN_INNER_URL = BK_PAAS_INNER_HOST + "/login"

    # V2环境 无该环境变量
    BK_COMPONENT_API_URL = None

else:  # V3
    # 蓝鲸PASS平台URL
    BK_PAAS_HOST = os.getenv("BK_PAAS2_URL", BK_URL).rstrip("/")

    # 用于 用户认证、用户信息获取 的蓝鲸主机
    BK_PAAS_INNER_HOST = os.getenv("BK_PAAS2_INNER_URL", BK_PAAS_HOST).rstrip("/")

    # 兼容component的APP_ID,APP_TOKEN
    APP_CODE = APP_ID = get_env_or_raise("BKPAAS_APP_ID")
    SECRET_KEY = APP_TOKEN = get_env_or_raise("BKPAAS_APP_SECRET")

    # 蓝鲸登陆页面
    BK_LOGIN_URL = os.getenv("BKPAAS_LOGIN_URL", "%s/login" % BK_PAAS_HOST).rstrip("/")
    BK_LOGIN_INNER_URL = os.getenv(
        "BK_LOGIN_INNER_URL", "%s/login" % BK_PAAS_INNER_HOST
    ).rstrip("/")

    # 容器化V3 ESB地址
    BK_COMPONENT_API_URL = os.getenv("BK_COMPONENT_API_URL")


# REMOTE_STATIC_URL
REMOTE_STATIC_URL = "%sremote/" % STATIC_URL



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import os

from django.conf import settings
from django.utils.log import configure_logging
from opentelemetry import trace
from opentelemetry.exporter.jaeger.thrift import JaegerExporter
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.sampling import _KNOWN_SAMPLERS

from blueapps.core.celery import celery_app
from blueapps.opentelemetry.constants import DEFAULT_LOGGING_TRACE_FORMAT
from blueapps.opentelemetry.instrumentor import BKAppInstrumentor
from blueapps.opentelemetry.metrics.celery import MetricsServerStep
from blueapps.opentelemetry.metrics.instrumentor import SaaSMetricsInstrumentor
from blueapps.opentelemetry.utils import inject_logging_trace_info
from blueapps.settings import blueapps_settings
from blueapps.opentelemetry.export import LazyBatchSpanProcessor


def setup_trace_config(
    service_name: str,
    bk_data_id: int,
    otel_sampler: str,
    otel_grpc_host: str,
    bk_data_token: str = "",
):
    span_processor = getattr(settings, "BLUEAPPS_BKAPP_OTEL_SPAN_PROCESSOR", LazyBatchSpanProcessor)
    service_name_handler = blueapps_settings.BKAPP_OTEL_SERVICE_NAME_HANDLER
    service_name = service_name_handler(service_name).get_service_name()
    if settings.ENVIRONMENT == "dev":
        # local environment, use jaeger as trace service
        # docker run -p 16686:16686 -p 6831:6831/udp jaegertracing/all-in-one
        trace.set_tracer_provider(TracerProvider(resource=Resource.create({SERVICE_NAME: service_name})))
        jaeger_exporter = JaegerExporter(agent_host_name="localhost", agent_port=6831, udp_split_oversized_batches=True)
        trace.get_tracer_provider().add_span_processor(span_processor(jaeger_exporter))
    else:
        # stage and prod environment, use bk_log as trace service
        trace.set_tracer_provider(
            tracer_provider=TracerProvider(
                resource=Resource.create(
                    {"service.name": service_name, "bk_data_id": bk_data_id, "bk.data.token": bk_data_token},
                ),
                sampler=_KNOWN_SAMPLERS[otel_sampler],
            )
        )
        otlp_exporter = OTLPSpanExporter(endpoint=otel_grpc_host)
        span_processor = span_processor(otlp_exporter)
        trace.get_tracer_provider().add_span_processor(span_processor)


def setup_by_settings():
    enable_paas_otel = True if getattr(settings, "OTEL_BK_DATA_TOKEN", "") else False
    enable_trace = os.getenv("ENABLE_TRACE", enable_paas_otel) or blueapps_settings.ENABLE_OTEL_TRACE
    if enable_trace:
        app_module_name = getattr(settings, "APP_MODULE_NAME", "")
        service_name = (
            os.getenv("BKAPP_OTEL_SERVICE_NAME")
            or getattr(settings, "BKAPP_OTEL_SERVICE_NAME", None)
            or (settings.APP_CODE + f"-{app_module_name}" if app_module_name else "")
        )
        bk_data_id = int(os.getenv("BKAPP_OTEL_BK_DATA_ID") or blueapps_settings.BKAPP_OTEL_BK_DATA_ID)
        otel_sampler = os.getenv("BKAPP_OTEL_SAMPLER") or blueapps_settings.BKAPP_OTEL_SAMPLER
        otel_grpc_host = os.getenv("BKAPP_OTEL_GRPC_HOST") or blueapps_settings.BKAPP_OTEL_GRPC_HOST
        otel_bk_data_token = os.getenv("BKAPP_OTEL_BK_DATA_TOKEN") or blueapps_settings.BKAPP_OTEL_BK_DATA_TOKEN
        setup_trace_config(service_name, bk_data_id, otel_sampler, otel_grpc_host, bk_data_token=otel_bk_data_token)
        BKAppInstrumentor().instrument()
        # 将Trace信息配置到项目日志中
        trace_format = blueapps_settings.BKAPP_OTEL_LOGGING_TRACE_FORMAT or getattr(
            blueapps_settings, "OTEL_LOGGING_TRACE_FORMAT", DEFAULT_LOGGING_TRACE_FORMAT
        )
        inject_logging_trace_info(settings.LOGGING, ("verbose",), trace_format)
        configure_logging(settings.LOGGING_CONFIG, settings.LOGGING)
    # metrics
    enable_metrics = os.getenv("ENABLE_METRICS") or blueapps_settings.ENABLE_OTEL_METRICS
    if enable_metrics:
        SaaSMetricsInstrumentor().instrument()
        celery_app.steps["worker"].add(MetricsServerStep)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from __future__ import absolute_import

import os
import re

from blueapps.conf import environ, get_settings_from_module
from blueapps.conf.database import get_default_database_config_dict

locals().update(get_settings_from_module(environ))

BASE_DIR = locals()["BASE_DIR"]
APP_CODE = locals()["APP_CODE"]

ROOT_URLCONF = "urls"

SITE_ID = 1

INSTALLED_APPS = (
    "bkoauth",
    # 框架自定义命令
    "blueapps.contrib.bk_commands",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # account app
    "blueapps.account",
)

MIDDLEWARE = (
    # request instance provider
    "blueapps.middleware.request_provider.RequestProvider",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    # 跨域检测中间件， 默认关闭
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
    "django.middleware.security.SecurityMiddleware",
    # 蓝鲸静态资源服务
    "whitenoise.middleware.WhiteNoiseMiddleware",
    # Auth middleware
    "blueapps.account.middlewares.RioLoginRequiredMiddleware",
    "blueapps.account.middlewares.WeixinLoginRequiredMiddleware",
    "blueapps.account.middlewares.LoginRequiredMiddleware",
    # exception middleware
    "blueapps.core.exceptions.middleware.AppExceptionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
)

DATABASES = {"default": get_default_database_config_dict(locals())}

# Cache

CACHES = {
    "db": {"BACKEND": "django.core.cache.backends.db.DatabaseCache", "LOCATION": "django_cache"},
    "login_db": {"BACKEND": "django.core.cache.backends.db.DatabaseCache", "LOCATION": "account_cache"},
    "dummy": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"},
    "locmem": {"BACKEND": "django.core.cache.backends.locmem.LocMemCache"},
}

CACHES["default"] = CACHES["dummy"]

# Template

MAKO_DIR_NAME = "mako_templates"

TEMPLATE_CONTEXT_PROCESSORS = [
    "django.template.context_processors.debug",
    "django.template.context_processors.request",
    "django.contrib.auth.context_processors.auth",
    "django.contrib.messages.context_processors.messages",
    "blueapps.template.context_processors.blue_settings",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": (os.path.join(BASE_DIR, "templates"),),
        "APP_DIRS": True,
        "OPTIONS": {"context_processors": TEMPLATE_CONTEXT_PROCESSORS},
    },
    {
        "BACKEND": "blueapps.template.backends.mako.MakoTemplates",
        "DIRS": (os.path.join(BASE_DIR, MAKO_DIR_NAME),),
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": TEMPLATE_CONTEXT_PROCESSORS,
            # mako templates cache, None means not using cache
            "module_directory": os.path.join(os.path.dirname(BASE_DIR), "templates_module", APP_CODE),
        },
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/
# 国际化配置
LOCALE_PATHS = (os.path.join(BASE_DIR, "locale"),)

ALLOWED_HOSTS = ["*"]
TIME_ZONE = "Asia/Shanghai"
LANGUAGE_CODE = "zh-hans"
USE_I18N = True
USE_L10N = True

LANGUAGES = (
    ("en", "English"),
    ("zh-hans", "简体中文"),
)

# Authentication & Authorization

SESSION_COOKIE_AGE = 60 * 60 * 24 * 7 * 2
SESSION_COOKIE_NAME = "_".join([APP_CODE, "sessionid"])
AUTH_USER_MODEL = "account.User"

AUTHENTICATION_BACKENDS = (
    "blueapps.account.backends.RioBackend",
    "blueapps.account.backends.WeixinBackend",
    "blueapps.account.backends.UserBackend",
)

RE_MOBILE = re.compile(r"Mobile|Android|iPhone|iPad|iPod", re.IGNORECASE)
RE_WECHAT = re.compile(r"MicroMessenger", re.IGNORECASE)

# CSRF Config
CSRF_COOKIE_NAME = APP_CODE + "_csrftoken"

# close celery hijack root logger
CELERYD_HIJACK_ROOT_LOGGER = False

# log_dir_prefix
LOG_DIR_PREFIX = "/app/v3logs/"

# 登录缓存时间配置, 单位秒（与django cache单位一致）
LOGIN_CACHE_EXPIRED = int(os.getenv("LOGIN_CACHE_EXPIRED", 60))

# CELERY与RabbitMQ增加60秒心跳设置项
BROKER_HEARTBEAT = 60

# OTEL TRACE
OTEL_BK_DATA_TOKEN = os.getenv("OTEL_BK_DATA_TOKEN", "")
OTEL_GRPC_HOST = os.getenv("OTEL_GRPC_URL", "")
OTEL_SAMPLER = os.getenv("OTEL_SAMPLER", "")

# APP模块名称
APP_MODULE_NAME = os.getenv("BKPAAS_APP_MODULE_NAME", "default")

# 是否开启 blueapps 内置数据表加密
BLUEAPPS_ENABLE_DB_ENCRYPTION = False



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""
from rest_framework import status
from rest_framework.renderers import JSONRenderer
from rest_framework.status import is_success

from blueapps.utils.request_provider import get_or_create_local_request_id


class APIRenderer(JSONRenderer):
    """
    统一的结构封装返回内容
    """

    SUCCESS_CODE = 0

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        统一处理返回数据
        """
        response = renderer_context["response"]
        req_id = get_or_create_local_request_id()

        res_data = {
            "result": True,
            "code": self.SUCCESS_CODE,
            "data": None,
            "message": None,
            "request_id": req_id,
            "trace_id": getattr(renderer_context.get("request"), "otel_trace_id", None),
        }

        if is_success(response.status_code):
            response.status_code = status.HTTP_200_OK
            res_data["data"] = data
            res_data["code"] = self.SUCCESS_CODE
            return super(APIRenderer, self).render(res_data, accepted_media_type, renderer_context)

        code = response.status_code
        message = response.data
        errors = response.data
        if isinstance(response.data, dict) and "code" in response.data:
            code = response.data["code"]
            message = self.pretty_dict(response.data["message"])
            errors = response.data["data"]

        res_data.update({"result": False, "code": code, "message": message, "errors": errors})

        return super(APIRenderer, self).render(res_data, accepted_media_type, renderer_context)

    def pretty_dict(self, dict_data):  # pylint: disable=no-self-use
        """
        将字典转为字符串返回
        格式: {key}: {value}
        """
        if not isinstance(dict_data, dict) or not dict_data:
            return dict_data
        res = []
        for key, value in dict_data.items():
            res.append(f"{key}: {value}")
        return "; ".join(res)



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import time

from django.http import JsonResponse
from django.middleware import csrf
from django.shortcuts import render

from blueapps.account.decorators import login_exempt
from blueapps.middleware.xss.decorators import escape_exempt


@login_exempt
def login_success(request):
    """
    弹框登录成功返回页面
    """
    return render(request, "account/login_success.html")


@login_exempt
def login_page(request):
    """
    跳转至固定页面，然后弹框登录
    """
    refer_url = request.GET.get("refer_url")

    context = {"refer_url": refer_url}
    return render(request, "account/login_page.html", context)


@login_exempt
@escape_exempt
def login_forbidden(request):
    """
    跳转至固定页面，拦截登录
    """
    hint_msg = request.GET.get("hint_msg")
    context = {"hint_msg": hint_msg}
    return render(request, "account/login_forbidden.html", context)


def send_code_view(request):
    ret = request.user.send_code()
    return JsonResponse(ret)


def get_user_info(request):

    return JsonResponse(
        {
            "code": 0,
            "data": {"id": request.user.id, "username": request.user.username, "timestamp": time.time()},
            "message": "ok",
        }
    )


def get_csrf_token(request):
    """
    前端获取csrf_token接口
    """
    csrf_token = csrf.get_token(request)
    return JsonResponse({"csrf_token": csrf_token})



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from django.http.request import split_domain_port, validate_host
from django.utils.module_loading import import_module

from blueapps.conf import settings


class UserAgentMiddleware(object):
    def process_request(self, request):
        request.is_mobile = lambda: bool(
            settings.RE_MOBILE.search(request.META.get("HTTP_USER_AGENT", ""))
        )

        request.is_rio = lambda: bool(
            request.META.get("HTTP_STAFFNAME", "")
            and getattr(settings, "RIO_TOKEN", None)
            and settings.RE_WECHAT.search(request.META.get("HTTP_USER_AGENT", ""))
        )

        request.is_wechat = lambda: bool(
            settings.RE_WECHAT.search(request.META.get("HTTP_USER_AGENT", ""))
            and not request.is_rio()
        )

        request.is_bk_jwt = lambda: bool(request.META.get("HTTP_X_BKAPI_JWT", ""))


class SiteUrlconfMiddleware(object):
    top_module = "conf.sites"

    def process_request(self, request):
        domain, _ = split_domain_port(request.get_host())

        for site in settings.SITES:
            site = site.copy()
            try:
                if validate_host(domain, site["HOSTS"]):
                    urlconf = ".".join([self.top_module, site["NAME"], "urls"])
                    import_module(urlconf)
                    break
            except ImportError:
                pass
            except Exception:  # pylint: disable=broad-except
                pass
        else:
            urlconf = settings.ROOT_URLCONF

        request.urlconf = urlconf


class SiteSettingsMiddleware(object):
    top_module = "conf.sites"

    def _enter(self, module):
        for key in dir(module):
            if not key.startswith("_") and key == key.upper():
                self._changes[key] = {}
                if hasattr(settings, key):
                    self._changes[key]["func"] = setattr
                    self._changes[key]["args"] = [key, getattr(settings, key)]
                else:
                    self._changes[key]["func"] = delattr
                    self._changes[key]["args"] = [key]

                setattr(settings, key, getattr(module, key))

    def _exit(self):
        for key in self._changes:
            self._changes[key]["func"](settings, *self._changes[key]["args"])

    def process_request(self, request):
        domain, _ = split_domain_port(request.get_host())

        self._changes = {}

        for site in settings.SITES:
            site = site.copy()
            try:
                if validate_host(domain, site["HOSTS"]):
                    site_settings = ".".join(
                        [self.top_module, site["NAME"], "settings"]
                    )
                    self._enter(import_module(site_settings))
                    break
            except ImportError:
                pass

    def process_response(self, request, response):
        self._exit()
        return response

    def process_exception(self, request, exception):
        self._exit()



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import threading

from opentelemetry.sdk.trace import ReadableSpan
from opentelemetry.sdk.trace.export import BatchSpanProcessor


class LazyBatchSpanProcessor(BatchSpanProcessor):
    def __init__(self, *args, **kwargs):
        super(LazyBatchSpanProcessor, self).__init__(*args, **kwargs)
        # 停止默认线程
        self.done = True
        with self.condition:
            self.condition.notify_all()
        self.worker_thread.join()
        self.done = False
        self.worker_thread = None

    def on_end(self, span: ReadableSpan) -> None:
        if self.worker_thread is None:
            self.worker_thread = threading.Thread(name=self.__class__.__name__, target=self.worker, daemon=True)
            self.worker_thread.start()
        super(LazyBatchSpanProcessor, self).on_end(span)

    def shutdown(self) -> None:
        # signal the worker thread to finish and then wait for it
        self.done = True
        with self.condition:
            self.condition.notify_all()
        if self.worker_thread:
            self.worker_thread.join()
        self.span_exporter.shutdown()
